package bank.model;


import static org.junit.Assert.*;

import java.math.BigDecimal;
import java.math.RoundingMode;

import org.drools.KnowledgeBase;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import bank.infrastructure.impl.StatefulServiceImpl;
import bank.util.DroolsHelper;


public class BankAppTests {
	
	private StatefulServiceImpl statefulService;
	private static KnowledgeBase knowledgeBase;

	@BeforeClass
	public static void setUpClass() throws Exception {
		knowledgeBase = DroolsHelper.createKnowledgeBase();
	}
	
	@Before
	public void initilize() throws Exception {
		statefulService = new StatefulServiceImpl(knowledgeBase);
	}
	
	@After
	public void terminate() {
		statefulService.terminate();
	}
	
	private Customer createCustomerWithAccount() {
		Customer customer = new Customer();
		customer.setAddress(new Address());
		Account account = new Account();
		customer.addAccount(account);
		account.setOwner(customer);
		return customer;
	}
	
	private Customer createCustomerWithAccounts() {
		Customer customer = new Customer();
		customer.setAddress(new Address());
		Account account1 = new Account();
		customer.addAccount(account1);
		account1.setOwner(customer);
		Account account2 = new Account();
		customer.addAccount(account2);
		account2.setOwner(customer);
		return customer;
	}
	
	@Test
	public void bonusInTheBalance() throws Exception {
		Customer customer = createCustomerWithAccounts();
		customer.getAccounts().get(0).setBalance(new BigDecimal(150));
		customer.getAccounts().get(1).setBalance(new BigDecimal(50));
		statefulService.insertOrUpdateRecursive(customer);
		statefulService.executeRules();
		assertEquals(BigDecimal.valueOf(150).setScale(2, RoundingMode.HALF_EVEN),
				customer.getAccounts().get(0).getBalance().setScale(2, RoundingMode.HALF_EVEN));
		assertEquals(BigDecimal.valueOf(52.5).setScale(2, RoundingMode.HALF_EVEN), 
				customer.getAccounts().get(1).getBalance().setScale(2, RoundingMode.HALF_EVEN));
		System.out.println("bonusInTheBalance -> " + statefulService.getMessages());
		assertTrue(statefulService.getMessages().contains("bonusInTheBalance"));
		assertTrue(statefulService.getMessages().contains("suggestInvestment"));
		assertFalse(statefulService.getMessages().contains("suggestLoan"));
	}
	
	@Test
	public void suggestInvestment() throws Exception {
		Customer customer = createCustomerWithAccount();
		customer.getAccounts().get(0).setBalance(new BigDecimal(300));
		statefulService.insertOrUpdateRecursive(customer);
		statefulService.executeRules();
		System.out.println("suggestInvestment -> " + statefulService.getMessages());
		assertTrue(statefulService.getMessages().contains("suggestInvestment"));
		assertFalse(statefulService.getMessages().contains("bonusInTheBalance"));
		assertFalse(statefulService.getMessages().contains("suggestLoan"));
	}
	
	@Test
	public void suggestLoan() throws Exception {
		Customer customer = createCustomerWithAccount();
		customer.getAccounts().get(0).setBalance(new BigDecimal(-100));
		statefulService.insertOrUpdateRecursive(customer);
		statefulService.executeRules();
		System.out.println("suggestLoan -> " + statefulService.getMessages());
		assertTrue(statefulService.getMessages().get(0).equals("bonusInTheBalance"));
		assertTrue(statefulService.getMessages().get(1).equals("suggestLoan"));
		assertFalse(statefulService.getMessages().contains("suggestInvestment"));
	}
	
	@Test
	public void doBonusAndSuggestInvestment() throws Exception {
		Customer customer = createCustomerWithAccount();
		customer.getAccounts().get(0).setBalance(new BigDecimal(98));
		statefulService.insertOrUpdateRecursive(customer);
		statefulService.executeRules();
		System.out.println("doBonusAndSuggestInvestment -> " + statefulService.getMessages());
		assertTrue(statefulService.getMessages().contains("bonusInTheBalance"));
		assertTrue(statefulService.getMessages().contains("suggestInvestment"));
		assertFalse(statefulService.getMessages().contains("suggestLoan"));
	}
	
	@Test
	public void insertTransaction() throws Exception {
		Customer customer = createCustomerWithAccounts();
		customer.setFirstName("Paulo");
		customer.setLastName("Farias");
		customer.getAccounts().get(0).setNumber(new Long(123));
		customer.getAccounts().get(0).setType(Account.Type.JOINT);
		customer.getAccounts().get(0).setBalance(new BigDecimal(7000));
		customer.getAccounts().get(1).setNumber(new Long(321));
		customer.getAccounts().get(1).setType(Account.Type.SAVINGS);
		statefulService.insertOrUpdateRecursive(customer);
		statefulService.executeRules();
		System.out.println("insertTransaction -> " + statefulService.getMessages());
		assertTrue(statefulService.getMessages().contains("insertTransaction"));
		assertFalse(statefulService.getMessages().contains("highTransaction123321"));
	}
	
	@Test
	public void insertTransactionWithHighAmount() throws Exception {
		Customer customer = createCustomerWithAccounts();
		customer.setFirstName("Paulo");
		customer.setLastName("Farias");
		customer.getAccounts().get(0).setNumber(new Long(123));
		customer.getAccounts().get(0).setType(Account.Type.JOINT);
		customer.getAccounts().get(0).setBalance(new BigDecimal(50000));
		customer.getAccounts().get(1).setNumber(new Long(321));
		customer.getAccounts().get(1).setType(Account.Type.SAVINGS);
		statefulService.insertOrUpdateRecursive(customer);
		statefulService.executeRules();
		System.out.println("insertTransactionWithHighAmount -> " + statefulService.getMessages());
		assertTrue(statefulService.getMessages().contains("insertTransaction"));
		assertTrue(statefulService.getMessages().contains("highTransaction123321"));
	}

}
